@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # mpeg.py - mpeg 1/2 file parser
 # -----------------------------------------------------------------------------
-# $Id: mpeg.py 3647 2008-10-25 19:52:16Z hmeine $
+# $Id: mpeg.py 4041 2009-05-22 14:59:59Z tack $
 #
 # -----------------------------------------------------------------------------
 # kaa-Metadata - Media Metadata for Python
@@ -389,8 +389,12 @@ class MPEG(core.AVContainer):
                 self.get_time = self.ReadSCRMpeg2
                 return offset + (ord(buffer[offset+13]) & 0x07) + 14
             else:
-                # WTF? Very strange
-                return None
+                # I have no idea what just happened, but for some DVB
+                # recordings done with mencoder this points to a
+                # PACK_PKT describing something odd. Returning 0 here
+                # (let's hope there are no extensions in the header)
+                # fixes it.
+                return 0
 
         if 0xC0 <= id <= 0xDF:
             # code for audio stream
@@ -685,7 +689,7 @@ class MPEG(core.AVContainer):
         file.seek(0, 0)
         try:
             header = struct.unpack('>LL', file.read(8))
-        except:
+        except (struct.error, IOError):
             return False
 
         if header[0] != 0x1B3:
@@ -735,9 +739,13 @@ class MPEG(core.AVContainer):
                 c += TS_PACKET_LENGTH
                 continue
 
+            pid = ((ord(buffer[c+1]) & 0x1f) << 8) | ord(buffer[c+2])
+
             tsid = ((ord(buffer[c+1]) & 0x3F) << 8) + ord(buffer[c+2])
             adapt = (ord(buffer[c+3]) & 0x30) >> 4
 
+            afc = (ord(buffer[c+3]) >> 4) & 3
+
             offset = 4
             if adapt & 0x02:
                 # meta info present, skip it for now
@@ -834,8 +842,12 @@ class MPEG(core.AVContainer):
         if not hasattr(self, 'filename') or not hasattr(self, 'start'):
             return None
 
+        length = os.stat(self.filename)[stat.ST_SIZE]
+        if length < self.__sample_size__:
+            return
+
         file = open(self.filename)
-        file.seek(os.stat(self.filename)[stat.ST_SIZE]-self.__sample_size__)
+        file.seek(length - self.__sample_size__)
         buffer = file.read(self.__sample_size__)
 
         end = None
